@@ -5,18 +5,45 @@ Eg. RFC 2426 (vCard)
 
 It is stupid in the sense that it treats the content purely as strings. No type
 conversion is attempted.
-
-Copyright, 2005: Max M <maxm@mxm.dk>
 """
-
 import re
-import icalendar.tools
 from types import TupleType, ListType
 from icalendar.caselessdict import CaselessDict
+import logging
+logger = logging.getLogger('icalendar')
 SequenceTypes = [TupleType, ListType]
 
 
-def foldline(text, lenght=75, newline='\r\n'):
+def escape_char(text):
+    """Format value according to iCalendar TEXT escaping rules.
+    """
+    # NOTE: ORDER MATTERS!
+    return text.replace('\N', '\n')\
+               .replace('\\', '\\\\')\
+               .replace(';', r'\;')\
+               .replace(',', r'\,')\
+               .replace('\r\n', r'\n')\
+               .replace('\n', r'\n')
+
+def unescape_char(text):
+    # NOTE: ORDER MATTERS!
+    return text.replace(r'\N', r'\n')\
+               .replace(r'\r\n', '\n')\
+               .replace(r'\n', '\n')\
+               .replace(r'\,', ',')\
+               .replace(r'\;', ';')\
+               .replace('\\\\', '\\')
+
+def tzinfo_from_dt(dt):
+    tzid = None
+    if hasattr(dt.tzinfo, 'zone'):
+        tzid = dt.tzinfo.zone # pytz implementation
+    elif hasattr(dt.tzinfo, 'tzname'):
+        tzid = dt.tzinfo.tzname(dt) # dateutil implementation
+    return tzid
+
+
+def foldline(text, length=75, newline='\r\n'):
     """Make a string folded per RFC5545 (each line must be less than 75 octets)
 
     >>> from icalendar.parser import foldline
@@ -27,22 +54,54 @@ def foldline(text, lenght=75, newline='\r\n'):
     ...             "Vestibulum convallis imperdiet dui posuere.")
     >>> foldline(longtext)
     ... # doctest: +NORMALIZE_WHITESPACE
-    'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vestibulum \\r\\n
-    convallis imperdiet dui posuere.'
-
-#    >>> uuu = u'alfdkadaskd'
-#    >>> foldline(uuu, length=3)
-#    u'alf\ndka\nd\xe4\xe4\n\xe4\xfc\xfc\nask\nd'
-
+    'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vestibulum conval\\r\\n
+    lis imperdiet dui posuere.'
     """
-    return newline.join(
-            icalendar.tools.wrap(text, lenght,
-                subsequent_indent=' ',
-                drop_whitespace=False,
-                break_long_words=True,
-                replace_whitespace=False
-                )
-            )
+    assert isinstance(text, str)
+#    text.decode('utf-8')  # try to decode, to be sure it's utf-8 or ASCII
+    l_line = len(text)
+    new_lines = []
+    start = 0
+    while True:
+        end = start + length - 1
+        chunk = text[start:end]
+        m = NEWLINE.search(chunk)
+        if m is not None and m.end() != l_line:
+            new_lines.append(text[start:start + m.start()])
+            start += m.end()
+            continue
+
+        if end >= l_line:
+            end = l_line
+        else:
+            # Check that we don't fold in the middle of a UTF-8 character:
+            # http://lists.osafoundation.org/pipermail/ietf-calsify/2006-August/001126.html
+            while True:
+                char_value = ord(text[end])
+                if char_value < 128 or char_value >= 192:
+                    # This is not in the middle of a UTF-8 character, so we
+                    # can fold here:
+                    break
+                else:
+                    end -= 1
+
+        # Recompute the chunk, since start or end may have changed.
+        chunk = text[start:end]
+        new_lines.append(chunk)
+        if end == l_line:
+            break  # Done
+        start = end
+    return (newline + ' ').join(new_lines).rstrip(' ')
+
+#    return newline.join(
+#            icalendar.tools.wrap(text, length,
+#                subsequent_indent=' ',
+#                drop_whitespace=False,
+#                break_long_words=True,
+#                replace_whitespace=False
+#                )
+#            )
+
 
 #################################################################
 # Property parameter stuff
@@ -199,6 +258,7 @@ class Parameters(CaselessDict):
         """
         return self.keys()
 
+### TODO?
 ### Later, when I get more time... need to finish this off now. The last majot thing missing.
 ###    def _encode(self, name, value, cond=1):
 ###        # internal, for conditional convertion of values.
@@ -226,7 +286,7 @@ class Parameters(CaselessDict):
         items.sort() # To make doctests work
         for key, value in items:
             value = paramVal(value)
-            result.append('%s=%s' % (key.upper(), value))
+            result.append('%s=%s' % (key.upper(), value.encode(DEFAULT_ENCODING)))
         return ';'.join(result)
 
 
@@ -282,17 +342,17 @@ class Contentline(str):
     A long line gets folded
     >>> c = Contentline(''.join(['123456789 ']*10))
     >>> c.to_ical()
-    '123456789 123456789 123456789 123456789 123456789 123456789 123456789 \\r\\n 123456789 123456789 123456789 '
+    '123456789 123456789 123456789 123456789 123456789 123456789 123456789 1234\\r\\n 56789 123456789 123456789'
 
     A folded line gets unfolded
     >>> c = Contentline.from_ical(c.to_ical())
     >>> c
-    '123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 '
+    '123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789'
 
     Newlines in a string get need to be preserved
     >>> c = Contentline('1234\\n\\n1234')
     >>> c.to_ical()
-    '1234\\n\\n1234'
+    '1234\\r\\n \\r\\n 1234'
 
     We do not fold within a UTF-8 character:
     >>> c = Contentline('This line has a UTF-8 character where it should be folded. Make sure it g\xc3\xabts folded before that character.')
@@ -341,10 +401,14 @@ class Contentline(str):
 
     A value can also be unicode
     >>> from icalendar.prop import vText
-    >>> parts = ('SUMMARY', Parameters(), vText(u'INternational char   '))
+    >>> parts = ('SUMMARY', Parameters(), vText(u'INternational char æ ø å'))
     >>> Contentline.from_parts(parts)
     'SUMMARY:INternational char \\xc3\\xa6 \\xc3\\xb8 \\xc3\\xa5'
 
+    'SUMMARY:INternational char \xc3\x83\xc2\xa6 \xc3\x83\xc2\xb8 \xc3\x83\xc2\xa5'
+
+    TODO: troubles with console and doctest encodings?
+
     Traversing could look like this.
     >>> name, params, vals = c.parts()
     >>> name
@@ -414,13 +478,18 @@ class Contentline(str):
         "Turns a tuple of parts into a content line"
         (name, params, values) = parts
         try:
-            if values and not isinstance(values, basestring):
-                if hasattr(values, 'to_ical'):
-                    values = values.to_ical()
+            if hasattr(values, 'to_ical'):
+                values = values.to_ical()
+            else:
+                values = vText(values).to_ical()
+            #elif isinstance(values, basestring):
+            #    values = escape_char(values)
+
             if params:
                 return Contentline('%s;%s:%s' % (name, params.to_ical(), values))
             return Contentline('%s:%s' %  (name, values))
-        except:
+        except Exception, e:
+            logger.error(str(e))
             raise ValueError(
                 'Property: %s Wrong values "%s" or "%s"' % (repr(name),
                                                             repr(params),
@@ -489,7 +558,7 @@ class Contentlines(list):
     Lets try appending it with a 100 charater wide string
     >>> c.append(Contentline(''.join(['123456789 ']*10)+'\\r\\n'))
     >>> c.to_ical()
-    'BEGIN:VEVENT\\r\\n\\r\\n123456789 123456789 123456789 123456789 123456789 123456789 123456789 \\r\\n 123456789 123456789 123456789 \\r\\n\\r\\n'
+    'BEGIN:VEVENT\\r\\n\\r\\n123456789 123456789 123456789 123456789 123456789 123456789 123456789 1234\\r\\n 56789 123456789 123456789 \\r\\n\\r\\n'
 
     Notice that there is an extra empty string in the end of the content lines.
     That is so they can be easily joined with: '\r\n'.join(contentlines)).
@@ -543,3 +612,5 @@ class Contentlines(list):
 #('UID', Parameters({}), '42')
 #('END', Parameters({}), 'VEVENT')
 #('END', Parameters({}), 'VCALENDAR')
+
+from icalendar.prop import vText
