@@ -51,6 +51,9 @@ from datetime import (
 from types import TupleType, ListType
 from icalendar.caselessdict import CaselessDict
 from icalendar.parser import Parameters
+from icalendar.parser import escape_char
+from icalendar.parser import unescape_char
+from icalendar.parser import tzinfo_from_dt
 
 
 SequenceTypes = [TupleType, ListType]
@@ -411,17 +414,19 @@ class vDDDTypes:
     >>> vDDDTypes(42)
     Traceback (most recent call last):
         ...
-    ValueError: You must use datetime, date or timedelta
+    ValueError: You must use datetime, date, timedelta or time
     """
 
     def __init__(self, dt):
         "Returns vDate from"
-        if type(dt) not in (datetime, date, timedelta):
-            raise ValueError ('You must use datetime, date or timedelta')
+        if type(dt) not in (datetime, date, timedelta, time):
+            raise ValueError ('You must use datetime, date, timedelta or time')
         if isinstance(dt, datetime):
             self.params = Parameters(dict(value='DATE-TIME'))
-        elif isinstance(dt, date): # isinstance(datetime_object, date) => True
+        elif isinstance(dt, date):
             self.params = Parameters(dict(value='DATE'))
+        elif isinstance(dt, time):
+            self.params = Parameters(dict(value='TIME'))
         self.dt = dt
 
     def to_ical(self):
@@ -432,6 +437,8 @@ class vDDDTypes:
             return vDate(dt).to_ical()
         elif isinstance(dt, timedelta):
             return vDuration(dt).to_ical()
+        elif isinstance(dt, time):
+            return vTime(dt).to_ical()
         else:
             raise ValueError('Unknown date type')
 
@@ -444,8 +451,11 @@ class vDDDTypes:
             return vDuration.from_ical(ical)
         try:
             return vDatetime.from_ical(ical, timezone=timezone)
-        except:
-            return vDate.from_ical(ical)
+        except ValueError:
+            try:
+                return vDate.from_ical(ical)
+            except ValueError:
+                return vTime.from_ical(ical)
 
     from_ical = staticmethod(from_ical)
 
@@ -457,6 +467,10 @@ class vDate:
     >>> vDate(d).to_ical()
     '20010101'
 
+    >>> d = date(1899, 1, 1)
+    >>> vDate(d).to_ical()
+    '18990101'
+
     >>> vDate.from_ical('20010102')
     datetime.date(2001, 1, 2)
 
@@ -473,7 +487,7 @@ class vDate:
         self.params = Parameters(dict(value='DATE'))
 
     def to_ical(self):
-        return self.dt.strftime("%Y%m%d")
+        return "%04d%02d%02d" % (self.dt.year, self.dt.month, self.dt.day)
 
     def from_ical(ical):
         "Parses the data format from ical text format"
@@ -514,6 +528,10 @@ class vDatetime:
     >>> vDatetime(dutc).to_ical()
     '20010101T123000Z'
 
+    >>> dutc = datetime(1899, 1,1, 12, 30, 0, tzinfo=pytz.utc)
+    >>> vDatetime(dutc).to_ical()
+    '18990101T123000Z'
+
     >>> vDatetime.from_ical('20010101T000000')
     datetime.datetime(2001, 1, 1, 0, 0)
 
@@ -547,12 +565,21 @@ class vDatetime:
 
     def to_ical(self):
         dt = self.dt
-        tzid = dt.tzinfo and dt.tzinfo.zone or None
+        tzid = tzinfo_from_dt(dt)
+
+        s = "%04d%02d%02dT%02d%02d%02d" % (
+            dt.year,
+            dt.month,
+            dt.day,
+            dt.hour,
+            dt.minute,
+            dt.second
+        )
         if tzid == 'UTC':
-            return dt.strftime("%Y%m%dT%H%M%SZ")
+            s += "Z"
         elif tzid:
             self.params.update({'TZID': tzid})
-        return dt.strftime("%Y%m%dT%H%M%S")
+        return s
 
     def from_ical(ical, timezone=None):
         """ Parses the data format from ical text format.
@@ -762,7 +789,8 @@ class vPeriod:
 
         self.params = Parameters()
         # set the timezone identifier
-        tzid = start.tzinfo and start.tzinfo.zone or None
+        # does not support different timezones for start and end
+        tzid = tzinfo_from_dt(start)
         if tzid:
             self.params['TZID'] = tzid
 
@@ -916,7 +944,7 @@ class vFrequency(str):
         try:
             return vFrequency(ical.upper())
         except:
-            raise ValueError, 'Expected weekday abbrevation, got: %s' % ical
+            raise ValueError, 'Expected frequency, got: %s' % ical
 
     from_ical = staticmethod(from_ical)
 
@@ -1100,33 +1128,17 @@ class vText(unicode):
         self.params = Parameters()
         return self
 
-    def escape(self):
-        """
-        Format value according to iCalendar TEXT escaping rules.
-        """
-        return (self.replace('\N', '\n')
-                    .replace('\\', '\\\\')
-                    .replace(';', r'\;')
-                    .replace(',', r'\,')
-                    .replace('\r\n', r'\n')
-                    .replace('\n', r'\n')
-                )
-
     def __repr__(self):
         return u"vText(%s)" % unicode.__repr__(self)
 
     def to_ical(self):
-        return self.escape().encode(self.encoding)
+        return escape_char(self).encode(self.encoding)
 
     def from_ical(ical):
-        "Parses the data format from ical text format"
+        """Parses the data format from ical text format.
+        """
         try:
-            ical = (ical.replace(r'\N', r'\n')
-                        .replace(r'\r\n', '\n')
-                        .replace(r'\n', '\n')
-                        .replace(r'\,', ',')
-                        .replace(r'\;', ';')
-                        .replace('\\\\', '\\'))
+            ical = unescape_char(ical)
             return ical.decode(vText.encoding, 'replace')
         except:
             raise ValueError, 'Expected ical text, got: %s' % ical
@@ -1134,9 +1146,8 @@ class vText(unicode):
     from_ical = staticmethod(from_ical)
 
 
-class vTime(time):
-    """A subclass of datetime, that renders itself in the iCalendar time
-    format.
+class vTime:
+    """Render and generates iCalendar time format.
 
     >>> dt = vTime(12, 30, 0)
     >>> dt.to_ical()
@@ -1152,13 +1163,17 @@ class vTime(time):
     ValueError: Expected time, got: 263000
     """
 
-    def __new__(cls, *args, **kwargs):
-        self = super(vTime, cls).__new__(cls, *args, **kwargs)
-        self.params = Parameters()
-        return self
+    def __init__(self, *args):
+        if len(args) == 1:
+            if not isinstance(args[0], (time, datetime)):
+                raise ValueError('Expected a datetime.time, got: %s' % args[0])
+            self.dt = args[0]
+        else:
+            self.dt = time(*args)
+        self.params = Parameters(dict(value='TIME'))
 
     def to_ical(self):
-        return self.strftime("%H%M%S")
+        return self.dt.strftime("%H%M%S")
 
     def from_ical(ical):
         "Parses the data format from ical text format"
